/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guihbciwizard.h"
#include "guifindmedium.h"
#include "keyfileopenhbci.h"


#include "list"
#include "string"
#include "math.h"
using namespace std;

#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qstring.h>
#include <qregexp.h>
#include <qvalidator.h>
#include <qprogressbar.h>
#include <qcheckbox.h>
#include <qfile.h>
#include <qlistbox.h>
#include <qprocess.h>
#include <qinputdialog.h>


/**
*this is the constructor. Reads bank list files and sets lists.  
*
*@param main 
*@param parent the parent dialog of this widget
*@param name widgets title
*@param hobit
*/
GuiHbciWizard::GuiHbciWizard( QWidget * parent, const char *name, HoBIT* hobit)
    :GuiHbciWizardBase(parent, name)
{
  _parent = parent;
  _hobit = hobit;
  _MediumDir = "";
  _MediumChanged = false;
  _NewKeyCreated = false;
  _KeyFound = false;   
  _ChangingPassword = false; 
  _HBCIVersion = 210;
  setModal(false);
  showExtension(false);

  cancelButton()->show();
  finishButton()->hide();
  setFinishEnabled(page( FINISH_PAGE ), true);
  lineEditPort->setValidator(new QIntValidator(0, 65535, lineEditPort));
  if(USE_LIVE_CD_FEATURES)
    buttonDialIn->show();
  else buttonDialIn->hide();  
    // button internationalisation
  cancelButton()->setText(tr("&Cancel"));
  finishButton()->setText(tr("&Finish"));
  nextButton()->setText(tr("&Next"));
  backButton()->setText(tr("&Back"));
  helpButton()->setText(tr("&Help"));

  _connectionTestSuccessful = false;
  _sendingKeysSuccessful = false;

  _hobit = hobit;

  _banks = _hobit->listOfKnownHbciBanks();//read bank list file
  list<HoBIT_Bank>::iterator bit;

  if (_banks.empty()) new QListViewItem(listViewBanks, tr("List with bankdata not found: ") + 
    MP_DIR()  + QString(MONEYPENNY_DATA_DIR) + "/" + QString(MONEYPENNY_HBCI_BANKDATA_FILE), "", "");

  for (bit = _banks.begin(); bit != _banks.end(); bit++)
  {
    HoBIT_Bank & bank =(*bit);
    new QListViewItem(  listViewBanks, 
			bank.getBankId(), 
			bank.getBankName(), 
			bank.getLocation(), 
			bank.getServer(),
			bank.getHBCIVersionString());
  }

  buttonSelectFile->hide();
  
  
  // timer for refreshing card reader list
  _timerFindCardReader = new QTimer(this);
  connect( _timerFindCardReader, SIGNAL(timeout()), SLOT(slotFindCardReaders()) );    
  
  widgetStack->raiseWidget( WELCOME_PAGE ); 
  
  QFont boldTitleFont = titleFont();
  boldTitleFont.setBold(true);
  setTitleFont(boldTitleFont); 
}


/** destructor */
  GuiHbciWizard::~GuiHbciWizard(){
  qDebug("Destructor GuiHbciWizard finished");
}


/**
* when user typed in bankid, look what bank it is and show it with name and code in lineedit
*/
void GuiHbciWizard::slotBankSelected()
{
  QListViewItem* it = listViewBanks->currentItem();
  QString bankCode = it->text(0);
  QString bankName = it->text(1);
  QString bankLocation = it->text(2);
  QString bankServer = it->text(3);
  QString HBCIVersion = it->text(4);

  lineEditBankCode->setText(bankCode);
  lineEditBankName->setText(bankName + ", " + bankLocation);
  lineEditBankServer->setText(bankServer);
  comboBoxHBCIVersion->setCurrentText (HBCIVersion);
  slotUpdateBankDataPage();  
}


/**
* function for changing CustomerId 
*/
void GuiHbciWizard::slotChangeCustomer()
{
  if ( lineEditUserId->text().isEmpty())
    setNextEnabled(currentPage(), false);
  else setNextEnabled(currentPage(), true);
}

/** Copies text from UserId to CustId
*/
void GuiHbciWizard::slotCopyUserId(){
  lineEditCustId->setText(lineEditUserId->text());
}

/**
* function for check if typed-in bank is existing
*/
void GuiHbciWizard::slotSearchBankId()
{
  bool bankFound = false;
  list<HoBIT_Bank>::iterator bit;
  for (bit = _banks.begin(); bit != _banks.end(); bit++)
  {
    QString bankId = (*bit).getBankId();

    if (bankId == lineEditBankCode->text())
    {
      lineEditBankName->setText((*bit).getBankName()+ ", " + (*bit).getLocation());
      lineEditBankServer->setText((*bit).getServer());
      int hbciVersion = (*bit).getHBCIVersion();
      setHbciComboBox( hbciVersion);
      lineEditBankCode->setPaletteBackgroundColor(green);
      lineEditBankName->setPaletteBackgroundColor(green);
      lineEditBankServer->setPaletteBackgroundColor(green);
      comboBoxHBCIVersion->setPaletteBackgroundColor(green);
      bankFound = true;
    }
  }
  if(!bankFound)
  {
    if (lineEditBankCode->text().length() >= 8){
      QColor orange(255, 164, 0);
      lineEditBankName->setText(tr("Could not find a bank with the code you input."));
      lineEditBankServer->setText(tr("May be, that your bank has no HBCI Server."));
      lineEditBankCode->setPaletteBackgroundColor(orange);
      lineEditBankName->setPaletteBackgroundColor(orange);
      lineEditBankServer->setPaletteBackgroundColor(orange);
      comboBoxHBCIVersion->setPaletteBackgroundColor(orange);
    }  
    else {
     lineEditBankName->setText("");
     lineEditBankServer->setText("");
     lineEditBankCode->setPaletteBackgroundColor(yellow);
     lineEditBankName->setPaletteBackgroundColor(yellow);
     lineEditBankServer->setPaletteBackgroundColor(yellow);
     comboBoxHBCIVersion->setPaletteBackgroundColor(yellow);
    } 
  }
  slotUpdateBankDataPage();  
}


/**
* function for slot new medium
*/
void GuiHbciWizard::slotNewMedium( int state)
{
  if(state != QButton::On)
   return;
  // stop searching for card readers
  _timerFindCardReader->stop();
  widgetStack->raiseWidget(0);
  buttonSelectFile->hide();
  _sendingKeysSuccessful = false;
  _NewKeyCreated = false;
  getKeyFileName(); //allow to change medium type
  lineEditMediumPath->setPaletteBackgroundColor(green);
  if(lineEditMediumPath->text() == "")
    lineEditMediumPath->setText(_MediumDir + "/" + "moneypenny.key");
  setNextEnabled(currentPage(), true); 
  textLabelHint->setText(tr("See below for your new key's name. You may edit the filename here. <br> Please proceed to the next page.")); 
}

/**
* function for slot existing medium
*/
void GuiHbciWizard::slotExistingMedium( int state)
{
  if(state != QButton::On)
   return;
  // stop searching for card readers
 _timerFindCardReader->stop();
  widgetStack->raiseWidget(0);
  buttonSelectFile->show();
  _sendingKeysSuccessful = true;
  labelSendKeys->setText(tr("You need not send the key again, if it has already been sent earlier."));
  getKeyFileName();
}

/**
* function for slot chip card  
*/
void GuiHbciWizard::slotChipCard( int state)
{
  if(state != QButton::On)
   return;
  // start the timer which searches card readers
   slotFindCardReaders();
 // _timerFindCardReader->start( 5000, FALSE ); !!!!
  widgetStack->raiseWidget(4);
}

/**
* function for change the medium type, called twice, once for every button 
*/
void GuiHbciWizard::slotChangeMediumType()
{
  if(isVisible()&& _MediumChanged)
   getKeyFileName();
  _MediumChanged = true;
}

/**
* function for choose keyfiles. Checks file format.
*/
void GuiHbciWizard::slotSearchFile()
{
  QString fileName;
  QString keyFiles = tr("HBCI key files (*.key)");
  QString userId = lineEditUserId->text();
  fileName = QFileDialog::getOpenFileName(lineEditMediumPath->text(),keyFiles,this);
  if (!fileName.isEmpty()){
    if (evaluateKeyFile(fileName))
      setNextEnabled(currentPage(), true);
   } 
  else setNextEnabled(currentPage(), false);       
}

/**
* function for a test connection to the bank. Creates a new user, if required. 
*/
void GuiHbciWizard::slotTestConnection()
{
  bool ok = true;
  // get information from the input mask
  string userid = lineEditUserId->text().ascii();
  string bankid = lineEditBankCode->text().ascii();
  string custid = lineEditCustId->text().ascii();
  if(custid.empty()) //user has cleared the customer id edit, simply copy userid 
   custid = userid;
  string server = lineEditBankServer->text().ascii();
  _HBCIVersion  = HoBIT_Bank::convertHBCIVersionString(comboBoxHBCIVersion->currentText());
  string mediumPath;
  string mediumtype;
  int country = MONEYPENNY_DEFAULT_COUNTRY;

  if (radioButtonChipCard->isOn())
  {
    mediumtype = HBCI_MEDIUM_TYPE_DDV;
    mediumPath = _CardNumber.latin1();
  }
  else if (radioButtonExistingMedium->isOn())
  {
    mediumtype = HBCI_MEDIUM_TYPE_RDH;
    mediumPath = lineEditMediumPath->text().ascii();
  }
  else if (radioButtonNewMedium->isOn())
  {
    mediumtype = HBCI_MEDIUM_TYPE_RDH;
    mediumPath = lineEditMediumPath->text().ascii();
  }
  else
  {
    QMessageBox::critical(this, tr("Data missing"), tr("A medium that contains the key must be specified."));
    return;
  }
  
  if (mediumtype == HBCI_MEDIUM_TYPE_DDV )
   ok =_hobit->isCardReaderOk();
   
  if(!ok){
     QMessageBox::critical(this, tr("Error"), 
      tr("Could not read chipcard.")+ "<br>"
      +tr("Please check cardreader and chipcard."));
     ok = false;
  }  
  
  if(ok){  // create user
   QString msg = "Creating user: mediumpath= " + mediumPath
                               + " mediumtype= " + mediumtype
                               + " country= " + QString::number(country)
                               + " bankid= " + bankid
                               + " HBCIVersion: " + QString::number(_HBCIVersion)
                               + " userid= " + userid
                               + " custid= " + custid
                               + " server= " + server;
   qDebug("GuiHBCIWizard::testConnection: %s", msg.latin1());
   ok = _hobit->createUser( mediumPath, 
                            mediumtype,
                            false,
                            country,
                            bankid,
                            _HBCIVersion, //2.2 does not work!
                            userid,
                            custid,
                            server) == 0;
   if (!ok){
     QMessageBox::critical(this, tr("Error"), 
      tr("Could not create user dataset.")+ "\n"
      +tr("Please check cardreader, chipcard and input data."));
   }
   
//   if(ok) 
//     _hobit->changeHbciVersion(_HBCIVersion);
 }// end create user block
 
 if (ok){   //if in RDH mode, run jobs GetKey and GetSystemId
    if (mediumtype == HBCI_MEDIUM_TYPE_RDH){
      _hobit->showLogText(tr("Adding job: Get bank's public key"));
      ok = _hobit->createJobGetKeys() == 0;
      _hobit->showLogText(tr("Executing job queue."));
      if(ok)
       ok = _hobit->executeJobQueue() == 0;
             
      if(ok ) // got the key, now try to get systemid and accounts. 
             // An activated user key is required for this to work
      {   
        _hobit->showLogText(tr("Adding job: System ID Renewal."));
        bool SystemIdOk = _hobit->createJobGetSystemId() == 0; //this may fail due to an unsent key 
        if(SystemIdOk){    
           _hobit->showLogText(tr("Executing job queue."));
           SystemIdOk = _hobit->executeJobQueue() == 0;
           string hash = _hobit->getBankFingerPrint(MONEYPENNY_DEFAULT_COUNTRY, bankid, custid);
           cerr << "Bank Hash: " <<  hash;
           cout << "Bank Hash: " <<  hash;
           textLabelBankHash->setText(QString(hash.c_str()));
        }
        if (!SystemIdOk ){ //this fails for a new key, however the messages are not required for a new key
          if(!_NewKeyCreated){
            _hobit->showLogText(tr("Executing JobGetSystemId failed. Key not yet activated?"));
            qDebug("JobGetSystemId failed. Key not yet activated?");
            QMessageBox::critical(this,tr("Error"), tr("Could not synchronize with the bank server.")+ "<br>"
             + tr("Most probably your public key was not yet sent or not activated by the bank."));
            labelSendKeys->setText(tr("Your key seems to be inactive.") + "<br>"
                                  +tr("Sending key is required, if you did not yet do so."));
           }
          _sendingKeysSuccessful = false;
        } 
        else{
          _sendingKeysSuccessful = true;
          labelSendKeys->setText(tr("Key is already known. Sending is not neccessary."));
        }  
      }//end systemId block
      if(_NewKeyCreated)
        labelSendKeys->setText(tr("Your new public key must be sent to the bank."));
    }//end HBCI_MEDIUM_TYPE_RDH
  
    // if in DDV card mode, run JobGetAccounts
    if (ok && mediumtype == HBCI_MEDIUM_TYPE_DDV){
       _hobit->showLogText(tr("Adding job: Get Accounts"));
       ok = _hobit->createJobGetAccounts() == 0;
       // now execute the job queue
       if(ok){
         _hobit->showLogText(tr("Executing job queue."));
         ok = _hobit->executeJobQueue() == 0;
       }
       if (!ok)
       {
         _hobit->showLogText(tr("Execution terminated with errors."));
         if(mediumtype == HBCI_MEDIUM_TYPE_DDV)
          QMessageBox::critical(this, tr("Error"), 
           tr("Job execution failed.") +" <br><b>" 
           +tr(" Does your chipcard contain HBCI-userid, bankid and bankserver?") + "</b><br>" 
           +tr(" If running from CD, refer to logfile /home/knoppix/moneypenny.log for details on card data.") + "</b><br>" 
           +tr(" Sorry, MoneyPenny does not write data to your card. Please ask your bank to add missing data."));
       }
    }//end if mediumtype == HBCI_MEDIUM_TYPE_DDV 
 }//end if create user ok

 //finally set the controls. Let the user continue, even if getting system id failed. He has to send the key.  
  if (ok){
    _hobit->showLogText(tr("Execution terminated successfully."));
    textLabelTestConnection->setText(tr("Connection test was successful."));
    textLabelTestConnection->setBackgroundColor(Qt::green);
    _connectionTestSuccessful = true;
    setNextEnabled(page( CONNECT_PAGE ), true);
  }
  if(!ok){
    textLabelTestConnection->setText(tr("failed"));
    textLabelTestConnection->setBackgroundColor(Qt::red);
    _connectionTestSuccessful = false;
    setNextEnabled(page( CONNECT_PAGE ), false);
  }     
  emit renewHoBIT();
  update();
}

/**
* Dialog for send the keys. User gets informations about the status of sending
*/
void GuiHbciWizard::slotSendKeys()
{
  HobitError error;
  bool hasErrors = false;

  // create a job to get systemid
  _hobit->showLogText(tr("Creating job: Send your public key"));
  error = _hobit->createJobSendKeys();
  hasErrors = (error != HobitErrorSuccess);

  // now execute the job queue
  _hobit->showLogText(tr("Executing job queue."));
  error = _hobit->executeJobQueue();
  hasErrors = (error != HobitErrorSuccess);
    
  if (hasErrors)
  {
    _hobit->showLogText(tr("Execution terminated with errors."));
    labelSendKeys->setText(tr("Sending public key failed."));
    labelSendKeys->setBackgroundColor(Qt::red);
    _sendingKeysSuccessful = false;
  }
  else{
    _hobit->showLogText(tr("Execution terminated successfully."));
    labelSendKeys->setText(tr("Sending key terminated successfully."));
    labelSendKeys->setBackgroundColor(Qt::green);
    _sendingKeysSuccessful = true;
  }

  pageChanged(); // set the controls
}

/** Fingerprints have been checked
*/
void GuiHbciWizard::slotFingerprintChecked(){
   if(checkBoxFingerprintOk->isChecked())
     checkBoxFingerprintOk->setPaletteBackgroundColor(green);
   setNextEnabled(page( FINGERPRINT_PAGE ), checkBoxFingerprintOk->isChecked());
}

/**
* Executed when the page changes 
*/
void GuiHbciWizard::pageChanged()
{
  emit wizardPageChanged(indexOf(currentPage())); //update page number and smallHelp in guiMain
  //any page
  if(radioButtonNewMedium->isOn()){
    setAppropriate(page(PASSWORD_PAGE), true); //skip the password and key generation page if no new medium is needed
    setAppropriate(page(ACCOUNTS_PAGE), false);
    }
  else {
     setAppropriate(page(PASSWORD_PAGE), false);
     setAppropriate(page(ACCOUNTS_PAGE), true);
  }
    
 //  Welcome page
  if (currentPage() == page( WELCOME_PAGE )){
    setNextEnabled(currentPage(), true);
    _MediumChanged = false;
    _NewKeyCreated = false;
    _KeyFound = false;   
    _ChangingPassword = false;
    cancelButton()->show();
  }
  
  //  internet setup page
  if (currentPage() == page( INTERNET_PAGE )){
    buttonTestInternetSetup->setFocus();
    setNextEnabled(currentPage(), true);
    cancelButton()->hide();
  }
  
  //  bank data page
  if (currentPage() == page( BANK_PAGE )){
    lineEditBankCode->setFocus();
    slotUpdateBankDataPage();
    setNextEnabled(currentPage(), true);
    cancelButton()->hide();
  }
  
  //  customer data page
  if (currentPage() == page( USER_PAGE ))
  {
    lineEditUserId->setFocus();
    slotChangeCustomer();
    cancelButton()->hide();    
  }

  // medium data page
  if (currentPage() == page( MEDIUM_PAGE ))
  {
    if ( !radioButtonNewMedium->isChecked() &&
         !radioButtonExistingMedium->isChecked() &&
         !radioButtonChipCard->isChecked())
      setNextEnabled(currentPage(), false);  
    else {
    getKeyFileName();
    cancelButton()->hide();
    setNextEnabled(currentPage(), _KeyFound || radioButtonNewMedium->isChecked() || radioButtonChipCard->isChecked()); 
  }
}
  // show data information page
  if (currentPage() == page( CHECK_PAGE ))
  {
    textLabelInformation->setText(getDataString());
    cancelButton()->hide();
   }

  //password and key generation page
  if (currentPage() == page( PASSWORD_PAGE )){ 
     if(_ChangingPassword){
       textLabelOldPassword->show();
       lineEditOldPassword->show();
       lineEditOldPassword->setFocus();
       setNextEnabled(currentPage() , false);
       cancelButton()->show();
     }
     else{
       textLabelOldPassword->hide();
       lineEditOldPassword->hide();
       lineEditPassword1->setFocus();
       setNextEnabled(currentPage() , _NewKeyCreated);
       cancelButton()->hide();
     }
  }
  
  //connection test page
  if (currentPage() == page( CONNECT_PAGE )){ 
    if (radioButtonChipCard->isChecked()){
      setAppropriate(page(PASSWORD_PAGE), false);
      setAppropriate(page(FINGERPRINT_PAGE), false);
      setAppropriate(page(SEND_KEY_PAGE), false);
      setAppropriate(page(FINISH_PAGE), true);
     _timerFindCardReader->stop();
     }
     else{
       if(radioButtonNewMedium->isChecked()){
          setAppropriate(page(PASSWORD_PAGE), true);
          setAppropriate(page(SEND_KEY_PAGE), true);
        } 
        setAppropriate(page(FINGERPRINT_PAGE), true);
     }

    if (_connectionTestSuccessful) {
      setNextEnabled(currentPage(), true);
    }
    else setNextEnabled(currentPage(), false);
    cancelButton()->hide();
  }
  // fingerprint page
  if (currentPage() == page( FINGERPRINT_PAGE )){ 
      setNextEnabled(currentPage(), checkBoxFingerprintOk->isChecked());
      cancelButton()->hide();
 }
  //send key page
  if (currentPage() == page( SEND_KEY_PAGE )){ 
    setNextEnabled(currentPage(), !_NewKeyCreated); //force sending a new key
    if (_sendingKeysSuccessful){  
      setNextEnabled(currentPage(), true); 
      labelSendKeys->setPaletteBackgroundColor(green);
    }  
    else{ 
      labelSendKeys->setPaletteBackgroundColor(yellow);
    }
   cancelButton()->hide();
 }

 //last page
  if (currentPage() == page( FINISH_PAGE ))
  {
    if(radioButtonChipCard->isChecked())
     textLabelCongratulations->setText(tr("<h1> Congratulations </h1>\
<p>\
You did succeed in setting up the securest bank communication system \
known up to now. You may now select an account and start homebanking. \
We recommend that you get your balance and transaction data as the first step.\
Please do not forget to remove your chipcard and to unplug the cardreader after you shutdown MonyeyPenny.\
</p><p>\
The MoneyPenny team wishes you a lot of joy while working with the program. \
</b>"));
   else
     textLabelCongratulations->setText(tr("h1> Congratulations </h1>\
<p>\
You did succeed in setting up the securest bank communication system \
known up to now. MoneyPenny is now ready to print the results of the \
communication setup process as a so called Ini-Letter. \
In order to activate your homebanking account, you have to sign this \
letter and send it to your bank. \
</p><p>\
HBCI homebanking will become useable as soon as your bank will activate \
your account. \
</p><p>\
The MoneyPenny team wishes you a lot of joy while working with the program. \
</b>"));
    
    setFinishEnabled(currentPage(), true);
    finishButton()->setText(tr("&Finish HBCI Setup"));
    cancelButton()->hide();
  }//end FINISH_PAGE
}


/**
* Executed when the user presses cancel button
*/
void GuiHbciWizard::reject()
{
  if(QMessageBox::question( _parent,
                         tr("Confirmation"),
                         tr("Cancel setup?<br> HBCI setup is not yet complete. Homebanking may not work. \
                             <br>Do you really want to cancel?"),
                         tr("&Yes"), tr("&No"),QString::null, 0, 1 ) == 0)
  emit aborted(); //this will run the main bank loop and restore old settings, if available
}

/**
* Executed when the user presses the accept button
*/
void GuiHbciWizard::accept()
{
  _hobit->saveEnvironment();
  GuiHbciWizardBase::accept();
  
  if (radioButtonChipCard->isOn())
  emit setupSuccessful(HBCI_MODE_DDV);
  else
  emit setupSuccessful(HBCI_MODE_RDH);
}

/**
* Call help widget passing the appropriate help file.
*/
void GuiHbciWizard::help()
{
  switch(indexOf(currentPage()))
  { 
    case WELCOME_PAGE  : emit helpRequest("hbci_welcome.html"); break;
    case INTERNET_PAGE : emit helpRequest("hbci_internet.html"); break;
    case BANK_PAGE     : emit helpRequest("hbci_bank.html"); break;
    case USER_PAGE     : emit helpRequest("hbci_user.html"); break;
    case MEDIUM_PAGE   : emit helpRequest("hbci_medium.html"); break;
    case CHECK_PAGE    : emit helpRequest("hbci_check.html"); break;
    case PASSWORD_PAGE : emit helpRequest("hbci_password.html"); break;
    case CONNECT_PAGE  : emit helpRequest("hbci_connect.html"); break;
    case FINGERPRINT_PAGE: emit helpRequest("hbci_fingerprint.html"); break;
    case SEND_KEY_PAGE : emit helpRequest("hbci_send_key.html"); break;
    case FINISH_PAGE   : emit helpRequest("hbci_finish.html"); break;
    default            : emit helpRequest("hbci_welcome.html"); break;
  }
}

/** Show the help page on password security
*/
void GuiHbciWizard::slotHelpPassword(){
 emit helpRequest("security.html");
}

/** Sets password bar according to log of number of possible passwords.
*/
void GuiHbciWizard::slotPasswordStrength(){
 QString password = lineEditPassword1->text();
 double nChars = 0.0; // use double to prevent numerical overflow
 int pos;
 QRegExp rx0( "[0-9]{1,20}");
 QRegExp rx1( "[a-z]{1,20}");
 QRegExp rx2( "[a-z, A-Z]{1,20}" );
 QRegExp rx3( "[a-z, A-Z, 0-9]{1,20}");
 QRegExpValidator v0( rx0, NULL );
 QRegExpValidator v1( rx1, NULL );
 QRegExpValidator v2( rx2, NULL );
 QRegExpValidator v3( rx3, NULL );
 if(v0.validate(password, pos) == QValidator::Acceptable)
   nChars = 10;
 else if(v1.validate(password, pos) == QValidator::Acceptable)
   nChars = 26;
 else if(v2.validate(password, pos) == QValidator::Acceptable)
   nChars = 52;
 else if(v3.validate(password, pos) == QValidator::Acceptable)
   nChars = 62;
 else if(password.length() > 0) 
   nChars = 128;
 
 double nPasswords = exp( log(nChars) *double(password.length()));
 int progress = int(log(nPasswords) / log(10.0) * 70.0 / 16.0); //70% for 10E16
 if(password.length() == 0)
   progress =0;
 progressBarStrength->setProgress(progress, 100); 
 progressBarStrength->setPaletteBackgroundColor(green);
 if(progress < 70)
  progressBarStrength->setPaletteBackgroundColor(yellow);
 if(progress < 40)
  progressBarStrength->setPaletteBackgroundColor(red);
}


/** Checks password and creates a new key if ok
*/
void GuiHbciWizard::slotCheckPassword(){
 setNextEnabled(currentPage(), false);
 if(lineEditPassword1->text() == lineEditPassword2->text() ){
   if(progressBarStrength->progress() > 40){ 
       if(_ChangingPassword)
         setNextEnabled(currentPage(), changeKeyfilePassword());
       else if (radioButtonNewMedium->isOn() && !_NewKeyCreated){   
         createNewKeyfile(); 
       }//end if !_NewKeyCreated
     }//end if progress > 40
     else  QMessageBox::critical(this, tr("Error"), tr("The password is too weak, please use a longer one."));         
 }   
 else QMessageBox::critical(this, tr("Input Error"), 
                     tr("The passwords in the two edits are not identical. <br> Please input again."));  
}

/** Creates a html formatted QString containing the data input.
*/
QString GuiHbciWizard::getDataString(){
    // get information from the input mask
    QString userid = QString(lineEditUserId->text());
    QString bankid = QString(lineEditBankCode->text());
    QString bankname = QString(lineEditBankName->text());
    QString custid = QString(lineEditCustId->text());
    QString server = QString(lineEditBankServer->text());
    QString mediumPath;

    QString mediumtype;

    if (radioButtonChipCard->isChecked())
    {
      mediumtype = tr("DDV-Chip Card");
      mediumPath = QString(lineEditMediumChipPath->text());
    }
    else if (radioButtonExistingMedium->isOn())
    {
      mediumtype = tr("RDH file on ");
      if (radioButtonFloppy->isChecked())
        mediumtype += tr("Floppy Disc");
      else mediumtype += tr("Usb-Memory-Stick");
      mediumPath = QString(lineEditMediumPath->text());
    }
    else if (radioButtonNewMedium->isChecked())
    {
      mediumtype = tr("RDH file on ");
      if (radioButtonFloppy->isChecked())
        mediumtype += tr("Floppy Disc");
      else mediumtype += tr("Usb-Memory-Stick");
      mediumPath = QString(lineEditMediumPath->text());          
    }
    else if (radioButtonChipCard->isChecked())
    {
      mediumtype = tr("Chip card");
      mediumPath = tr("Card reader ");          
    }
    
    QString data;
    data =  tr("Customer ID: "); data += custid; data += "<br>";
    data += tr("User Id: "); data += userid; data += "<br> <br>";
    data += tr("At bank: "); data += bankname; data += "<br>";
    data += tr("Bank ID: "); data += bankid; data += "<br>";
    data += tr("Server address: "); data += server; data += "<br><br>";
    data += tr("Medium used for keys: "); data += mediumtype; data += "<br>";
    if (radioButtonChipCard->isChecked()){
      data += tr("Card reader: ") + _CardReaderType + "<br>";
    }
    else {
      data += tr("File path to medium: ") + mediumPath + "<br>";
    }
    return data;
}

/** Updates the state of the controls on bank data page.
*/
void GuiHbciWizard::slotUpdateBankDataPage(){
    if ( lineEditBankCode->text().isEmpty() ||
         lineEditBankName->text().isEmpty() ||
         lineEditBankServer->text().isEmpty())
    {
      setNextEnabled(currentPage(), false);
    }
    else setNextEnabled(currentPage(), true);
}

/** Sets the corresponding radiobutton.(Floppy or Usb)
*/
void GuiHbciWizard::setStorageType(StorageType storage){
 switch(storage){
   case STORAGE_USB: radioButtonUsb->setChecked(true);break; //calls slotChangeMediumtype via connect
   case STORAGE_FLOPPY: radioButtonFloppy->setChecked(true); break; //calls slotChangeMediumtype
   default: radioButtonFloppy->setChecked(true); break; //calls slotChangeMediumtype
 }
 _MediumChanged = false; //reset flag. it has been set by slotChangeMediumtype
}

/** Write property for _MediumDir
*/
void GuiHbciWizard::setMediumDir(const QString & dir){
  _MediumDir = dir;
  lineEditMediumPath->setText(_MediumDir);
}

/** Searches for an existing key file or asks the user for a name.
*/
void GuiHbciWizard::getKeyFileName(){
    QString absFileName;
    QString fileName;
    _KeyFound = false;
    
    if(_MediumChanged){
      GuiFindMedium guifindmedium(this);
       if(guifindmedium.exec() == QDialog::Accepted){
        _MediumDir = guifindmedium.getCryptDirName();
      }  
    }  
    if((lineEditMediumPath->text() == "") || radioButtonNewMedium->isChecked()){
     absFileName = _MediumDir + "/moneypenny.key";
      lineEditMediumPath->setText(absFileName);
    }  
    else 
     absFileName = lineEditMediumPath->text();
    
    if(radioButtonExistingMedium->isChecked()){ //search for existing key
       QFile keyFile(absFileName);
       if(absFileName.endsWith(".key") && keyFile.exists()) //key file with default name found 
         _KeyFound= true;
       else{//search for any key file
         QFileInfo fileInfo(keyFile);
         if(! fileInfo.isDir())
          absFileName = fileInfo.dirPath();
         QDir dir(absFileName);
         QStringList fileList = dir.entryList();
         for(uint nFile = 0; nFile < fileList.count(); nFile++){
          fileName = fileList[nFile];
          if(fileName.find(".key") > 0){
           absFileName = _MediumDir + "/" + fileName;
            _KeyFound = true;
            break;
          }//end if
         }//end for   
       }//end search for key      
       
       if(_KeyFound){
          if(evaluateKeyFile(absFileName)){
            setNextEnabled(currentPage(), true);
            lineEditMediumPath->setPaletteBackgroundColor(green);
            textLabelHint->setText(tr("<br><br>Your key has been found. Please go to the next page. "));
          }
          else setNextEnabled(currentPage(), false);
       }
       else{     
        QMessageBox::information(this, tr("Information"), tr("Could not find a *.key file in directory: ") + _MediumDir + tr("\n Please search manually."));
        textLabelHint->setText(tr(
"Hint:<br>\
MoneyPenny is running on Linux. There are no drive letters.\
/ is used instead of \\ in the path.  Please try the following paths:<br>\
For floppy disk medium try /mnt/floppy or /media/floppy, for USB stick medium try /mnt/sda1 or /mnt/usb or /media/sda1 or/media/usb"));
       absFileName = _MediumDir;
       } 
       lineEditMediumPath->setText(absFileName);
    }//search for key   
}

/** Checks the Internet setup. Code is copied from guiConnections
*/
void GuiHbciWizard::slotTestInternetSetup(){
 ConnectionState state;
 emit testInternet(lineEditUrl->text(), lineEditPort->text().toInt(), &state); //connected to guiMain::slotTestInternet
 switch (state){
  case ConnectionWasOk:{
         labelInternetState->setText(tr("Status: Internet setup ok."));
         labelInternetState->setPaletteBackgroundColor(green);
         break;
        }
  case ConnectionFailed:{
        labelInternetState->setText(tr("Status: Internet setup is not ok. Cables connected?"));
        labelInternetState->setPaletteBackgroundColor(red);
        break;
       }
  default:{ 
       labelInternetState->setText(tr("Status: Internet setup not yet tested."));
       labelInternetState->setPaletteBackgroundColor(yellow);
       break;
     }
 }
}

/** Calls Internet setup program.
*/
void GuiHbciWizard::slotInternetSetup(){
 emit setupInternet(); //connected to guiMain::slotHardwareConfig
}


 /** show log widget
  */
  void GuiHbciWizard::slotShowDetails(){
    emit showLog(LOG_WIDGET);
  }

  
  
  
/** reads the user's previous configuration from the OpenHbci database */  
void GuiHbciWizard::evaluateStoredData(){
  
  Pointer<Bank> bank = _hobit->getBank();
  Pointer<User> user = _hobit->getUser();
  Pointer<Customer> customer = _hobit->getCustomer(); 
  Pointer<Medium> medium = _hobit->getMedium();
  
  QString mediumName;
  if(medium.isValid()){
    mediumName = medium.ref().mediumName().c_str();
    if(medium.ref().securityMode() == HBCI_SECURITY_RDH){
      qDebug("Using keyfile: %s", mediumName.latin1());
      lineEditMediumPath->setText(mediumName);   
      setAppropriate(page(PASSWORD_PAGE), false);     
      radioButtonExistingMedium->setChecked(true);
    }  
    else
      radioButtonChipCard->setChecked(true);  
  }
  else //no valid medium available
    radioButtonNewMedium->setChecked(true);
  
  if (bank.isValid()){
    lineEditBankCode->setText(bank.ref().bankCode().c_str());
    lineEditBankName->setText(bank.ref().name().c_str());
    lineEditBankServer->setText(bank.ref().address().c_str());
    
    int hbciVersion = bank.ref().hbciVersion();
    setHbciComboBox(hbciVersion);
    list<HoBIT_Bank>::iterator bit;
    bool found = false;
    for (bit = _banks.begin(); (bit != _banks.end()) && (!found); bit++)
    {
      QString bankId = (*bit).getBankId();
  
      if (bankId == lineEditBankCode->text()){
        lineEditBankName->setText((*bit).getBankName());
         found = true; // stop it 
        }  
     }
     
    if (user.isValid()){
      lineEditUserId->setText(user.ref().userId().c_str()); //this copies text to lineEditCustId via slotCopyUserId
      lineEditMediumPath->setText(mediumName);      
        
      if (customer.isValid()){
        lineEditCustId->setText(customer.ref().custId().c_str());  
        if(_hobit->listAccounts().empty()) //show the account error message and the skip button
          textLabelWelcome->setText(tr(
//Do not indent this multiline string          
"<font size=\"+1\"><h2>Welcome to MoneyPenny. </h2>\
Your configuration data have been found and read.\
However, there are no accounts in your configuration data.\
If you use a keyfile security medium (RDH,)then\
most probably your key was not yet activated by the bank,\
when you used MoneyPenny for the last time.<br>\
To obtain the account list from your bank,\
send the Ini-Letter to your bank and ask them to activate your key.\
Then run this wizard again.<br>\
Most banks send the account list during the connection test.\
However, some banks do not send account data.\
In this case, you will have to fill in the account form of this wizard yourself.\
</font>" ));
        }//end if customer is valid
        else textLabelWelcome->setText(tr(
//Do not indent this multiline string          
"<font size=\"+1\"><h2>Welcome to MoneyPenny. </h2>\
 Your configuration data have been found and read.\
 However, there seems to be a problem with the customer data.\
 We suggest, that you run this wizard again.\
 </font>"
                                         ));
    }//end if user is valid
  }//end if bank is valid
}


/** Reads /etc/chipcard.conf and lists all card readers found*/
void GuiHbciWizard::slotFindCardReaders(){

QString path  = getenv("HOME");
path += "/";
path += MONEYPENNY_MAIN_DIR;
path += "/";
uint nLine = 0;

if( !(_hobit->isChipcardDaemonRunning() && system("chipcard-tool list > " + path + "cardreaders.lst") == 0)){
 QMessageBox::critical(this, tr("Error"), 
 tr("Can not access cardreader.") + "<br>"
 + tr("Your libchipcard installation is not ok. Executing chipcard-tool failed."));
 return;
}
if (QFile::exists(path + "cardreaders.lst")){  
   labelReaders->clear();
   QString txt = "";
   
   QFile file(path + "cardreaders.lst");
       if ( file.open( IO_ReadOnly ) ) {
           QTextStream stream( &file );
           QString line;
           while ( !stream.atEnd() ) {
               nLine++;
               line = stream.readLine();
               txt += line + "\n"; // add '\n'
           }
           labelReaders->setText(txt);
           _CardReaderType = line;
        }  
        file.close();
   }
  if(nLine > 0){
    int country;
    QString bankId, userId, server;
    if(_hobit->readCardData(country, bankId, userId, server, _CardNumber)> 0){
      lineEditBankCode->setText(bankId);
      slotSearchBankId(); //hopefully the bank is in the list, so try to get data from there
      lineEditUserId->setText(userId);
      QMessageBox::information(this,tr("Information"), 
      tr("A set of user data has been read from your card and filled in the edit fields on the next pages.")+ "<br>" +
      tr("Please check the data versus the data you got from your bank in written form."));
    }
    else
     QMessageBox::information(this,tr("Information"), 
     tr("Could not read user data from your chipcard.") + "<br>"+
     tr("Please input required data into the edit fields on the next pages manually.")+
     tr("However, the data on your chipcard must be completed by your bank, before you can use MoneyPenny."));

    _sendingKeysSuccessful = true;
    setNextEnabled(currentPage(), true); 
    labelStatus->setText(tr("Using a DDV-chipcard from your Bank"));
    labelSendKeys->setText(tr("Key is already known. Sending is not neccessary."));
  }
  else{
    labelReaders->setText(tr("No cardreader was found. Can not continue.") + "<br>"
    + tr("Did you install libchipcard and the cardreader's driver?")); 
    setNextEnabled(currentPage(), false);
  }  
}

/** Opens card reader configuration tool */
void GuiHbciWizard::configureCardReader(){
  QProcess process(this);
  process.addArgument("kdesu");
  process.addArgument("kcardsetup");
  process.start();
}


  /** set up account list */
  bool GuiHbciWizard::findAccounts(){
	
   bool found = false;
   listBoxAccounts->clear();

    list<Pointer<Account> > accounts = _hobit->listAccounts();
    list<Pointer<Account> >::iterator  accit;

  if (accounts.empty()){
          listBoxAccounts->insertItem(tr("No accounts found"));
	 }
  else{ 
  	for (accit=accounts.begin(); accit!=accounts.end(); accit++){
             string accountId = (*accit).ref().accountId();
             listBoxAccounts->insertItem( QString(accountId.c_str()) );
        }
        found = true;
      }
 
 return found;
  }


/**
  *raises the add account widget 
  */
  void GuiHbciWizard::slotAddAccount(){
   emit addAccount();	
  }

  
void GuiHbciWizard::updateAccountList(){
  if (findAccounts()){
      textLabelAccounts->setText(tr("MoneyPenny has retrieved a list of your accounts from the bank server: "));
      pushButtonAddAccount->hide();
  }
}



/** Changes the password for the hbci communication. Only valid for RDH files.
*/
void GuiHbciWizard::changePassword(){
  HBCI_MODE mode = _hobit->getHbciMode();
  switch (mode){
    case HBCI_MODE_DDV:{
               QMessageBox::information(this, tr("Sorry"), tr("Can not change the PIN of your chipcard." ));
               break;
             }
    case HBCI_MODE_RDH:{
               _ChangingPassword = true;
               setAppropriate(page(PASSWORD_PAGE), true);
               setAppropriate(page(CHECK_PAGE), false);
               showPage(page(PASSWORD_PAGE));
               pageChanged();
               break;        
             }
    default: {
               QMessageBox::critical(this, tr("Error"), tr("No valid security mode has been set. Please run HBCI-setup first." ));
               break;
             }
          
  }
}

/** Create a new keyfile.
*/
void GuiHbciWizard::createNewKeyfile(){
  //check for file errors
  QMessageBox::information(this, tr("Information"), tr("A new security medium will be created now"));
  QString fileName = lineEditMediumPath->text();
  QFileInfo fileInfo(fileName);
  QString dirName = fileInfo.dirPath();
  QFileInfo dirInfo(dirName);
  if(!dirInfo.exists()){
   QMessageBox::critical(this, tr("Error"), tr("Directory: ") + dirName +
   tr(" Does not exist.<br>Please go to page before, select an existing directory and try again."));
   return;
  }
  if(fileInfo.isDir())
    fileName += dirName + "/" + lineEditUserId->text() + ".key";
  fileInfo.setFile(fileName);  
  if(fileInfo.exists()){
   if(QMessageBox::warning(this, tr("Warning"), tr("Key file exists.<br>\
      If you overwrite your key, you must send a new ini-letter to your bank, in order to activate the new key.<br>\
      <b>Homebanking with the new key does not work before the bank has activated it.</b>"),
      tr("Keep existing key and old password"), // button0 text
      tr("Overwrite existing key")) //button1 text
     == 0){ // keep button was clicked
         _NewKeyCreated = true;
         setNextEnabled(currentPage(), true);
         QMessageBox::information(this, tr("Information"), 
         tr("Using existing key file. Please proceed to next page.")); 
         return;
     }
     else if(!fileInfo.isWritable()){
      QMessageBox::critical(this, tr("Error"), tr("No write permission to file: ") + lineEditMediumPath->text() +
      tr("<br>Please make medium writable and try again."));
      return;
     }
  }//end if exists
  else if(!dirInfo.isWritable()){
    QMessageBox::critical(this, tr("Error"), tr("No write permission to directory: ") + dirName +
    tr("<br>Please make medium writable and try again."));
    return;
  }
 
  string userid = lineEditUserId->text().ascii();
  string custid = lineEditCustId->text().ascii();
  string bankid = lineEditBankCode->text().ascii();
  _HBCIVersion  = HoBIT_Bank::convertHBCIVersionString(comboBoxHBCIVersion->currentText());
  string mediumtype = HBCI_MEDIUM_TYPE_RDH;
  string mediumpath = lineEditMediumPath->text().ascii();
  string server = lineEditBankServer->text().ascii();
  int country = MONEYPENNY_DEFAULT_COUNTRY; //to be replaced for international use
   
  HobitError err = _hobit->createNewMedium( 
     mediumtype, false, country, bankid, userid, mediumpath, lineEditPassword1->text());
 
 // create user so that data will be found on reboot, even if user cancels further setup
 _hobit->createUser(mediumpath,
                    mediumtype,
                    false,
                    country,
                    bankid,
                     _HBCIVersion,
                    custid,
                    custid,
                    server);
  
  if (err == HobitErrorSuccess) {    
    _hobit->saveEnvironment();  
    _NewKeyCreated = true;
    setNextEnabled(currentPage(), true); 
    QMessageBox::information(this, tr("Information"), tr("The security file has been created successfully"));       
     emit renewHoBIT();
    }
  else QMessageBox::critical(this, tr("Error"), tr("The security file could not be created"));         
}

/** Changes password of current keyfile.
*/
bool GuiHbciWizard::changeKeyfilePassword(){
  if(QMessageBox::question(this, tr("Confirmation"), tr("You are about to change the password of the current key file.") +
                             tr("\nDo you really want to change the password?"), 
                             QMessageBox::No| QMessageBox::Escape , QMessageBox::Yes, QMessageBox::NoButton )
                             == QMessageBox::Yes) 
  {
     QString oldPassword = lineEditOldPassword->text();
     QString newPassword = lineEditPassword1->text();
     QString filename = lineEditMediumPath->text();
     KeyFileOpenHbci keyfile(filename, oldPassword);
     if(!keyfile.readFile()){
       QMessageBox::critical(this, tr("Error"), tr("Could not read keyfile: ") + filename + tr("\n Wrong password or wrong file format?"));
       return false;
     }
     else{
       if(keyfile.changePassword(newPassword)){
         QMessageBox::information(this, tr("Information"),tr("Your HBCI password has been changed."));
         emit(renewHoBIT()); //make shure, that any old key data are deleted
         return true;
       }
       else{
         QMessageBox::critical(this, tr("Error"), tr("Could not change password. No write access to keyfile?"));
         return false;
       }// end final error
     }// end if OK 
  }//end confirmed
  else return false;
}

/** Fires signal dialIn.
*/
void GuiHbciWizard::slotDialIn(){
 emit dialIn();
}

/** Sets hbci combo.
*/
void GuiHbciWizard::setHbciComboBox(int hbciVersion){
      switch(hbciVersion){
       case 201: comboBoxHBCIVersion->setCurrentItem(1); break;
       case 210: comboBoxHBCIVersion->setCurrentItem(2); break;
       case 220: comboBoxHBCIVersion->setCurrentItem(3); break; 
       default:  comboBoxHBCIVersion->setCurrentItem(1); break;    
      }
}

/**Reads the keyfile an puts data into edits.
*/
bool GuiHbciWizard::evaluateKeyFile(const QString & fileName){
    QString password = QInputDialog::getText
       (tr("Import keyfile"), tr("Please input the password for your existing keyfile."), QLineEdit::Password);
    KeyFileOpenHbci keyFile(fileName, password);
    if(keyFile.readFile()){
        QFileInfo info (fileName);
	QString newFileName = info.fileName(); //strip dir
	newFileName = _MediumDir + "/" + newFileName;
	qDebug("Using Keyfile: %s",newFileName.latin1());
	keyFile.writeFile(newFileName);
	lineEditMediumPath->setPaletteBackgroundColor(green);
	lineEditMediumPath->setText(newFileName);
	setNextEnabled(currentPage(), true);
        lineEditBankCode->setText(keyFile.getBankCode());
        lineEditUserId->setText(keyFile.getUserId());
        return true;
   }
   else{
     QMessageBox::critical(this, tr("Error"),tr("Could not read keyfile. Wrong password or unsupported file format."));
     return false;  
   }  
}

