/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/****************************************************************************
** $Id: guihelpwindow.cpp,v 1.5 2006/02/08 14:26:51 hbci Exp $
**
** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include "guihelpwindow.h"
#include <qstatusbar.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qiconset.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstylesheet.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qapplication.h>
#include <qcombobox.h>
#include <qevent.h>
#include <qlineedit.h>
#include <qobjectlist.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qprinter.h>
#include <qsimplerichtext.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <ctype.h>


/**
*constructor of GuiHelpWindow
*
*@param home_ QString
*@param _path QStringList
*@param parent pointer on current QWidget
*@param name name of the Widget
*/
GuiHelpWindow::GuiHelpWindow(QWidget* parent, const char *name, const QString& home_, const QStringList& _path )
    : GuiHelpWindowBase( parent, name )
{
    readHistory();
    readBookmarks();

    browser->mimeSourceFactory()->setFilePath( _path );
    connect( browser, SIGNAL( sourceChanged(const QString& ) ),
             this, SLOT( sourceChanged( const QString&) ) );

    pathCombo->insertItem( home_ );
  
    if ( !home_.isEmpty() )
        browser->setSource( home_ );
    qDebug("Help directory: %s", home_.latin1());    

    QPopupMenu* file = new QPopupMenu( this );
    file->insertItem( tr("&Open File"), this, SLOT( openFile() ), CTRL+Key_O );
    file->insertItem( tr("&Print"), this, SLOT( print() ), CTRL+Key_P );
    file->insertSeparator();
    file->insertItem( tr("&Close"), this, SLOT( close() ), CTRL+Key_Q );
    file->insertItem( tr("E&xit"), qApp, SLOT( closeAllWindows() ), CTRL+Key_X );

    // The same three icons are used twice each.
    QIconSet icon_back( QPixmap("back.xpm") );
    QIconSet icon_forward( QPixmap("forward.xpm") );
    QIconSet icon_home( QPixmap("home.xpm") );

    QPopupMenu* go = new QPopupMenu( this );
    backwardId = go->insertItem( icon_back,
                                 tr("&Backward"), browser, SLOT( backward() ),
                                 CTRL+Key_Left );
    forwardId = go->insertItem( icon_forward,
                                tr("&Forward"), browser, SLOT( forward() ),
                                CTRL+Key_Right );
    go->insertItem( icon_home, tr("&Home"), browser, SLOT( home() ) );

    //QPopupMenu* help = new QPopupMenu( this );
    
    hist = new QPopupMenu( this );
    QStringList::Iterator it = history.begin();
    for ( ; it != history.end(); ++it )
        mHistory[ hist->insertItem( *it ) ] = *it;
    connect( hist, SIGNAL( activated( int ) ),
             this, SLOT( histChosen( int ) ) );

    bookm = new QPopupMenu( this );
    bookm->insertItem( tr( "Add Bookmark" ), this, SLOT( addBookmark() ) );
    bookm->insertSeparator();

    QStringList::Iterator it2 = bookmarks.begin();
    for ( ; it2 != bookmarks.end(); ++it2 )
        mBookmarks[ bookm->insertItem( *it2 ) ] = *it2;
    connect( bookm, SIGNAL( activated( int ) ),
             this, SLOT( bookmChosen( int ) ) );

   browser->setFocus();

}

/**
*function for a source changing
*@param url QString with the location of the new source
*/
void GuiHelpWindow::sourceChanged( const QString& url )
{
    if ( !url.isEmpty() && pathCombo ) {
        bool exists = FALSE;
        int i;
        for ( i = 0; i < pathCombo->count(); ++i ) {
            if ( pathCombo->text( i ) == url ) {
                exists = TRUE;
                break;
            }
        }
        if ( !exists ) {
            pathCombo->insertItem( url, 0 );
            pathCombo->setCurrentItem( 0 );
            mHistory[ hist->insertItem( url ) ] = url;
        } else
            pathCombo->setCurrentItem( i );
    }
}

/**
*destructor of GuiHelpWindow
*/
GuiHelpWindow::~GuiHelpWindow()
{
 /*   qDebug("Destructor GuiHelpWindow called");
    history =  mHistory.values();

    QFile f( QDir::currentDirPath() + "/.history" );
    f.open( IO_WriteOnly );
    QDataStream s( &f );
    s << history;
    f.close();

    bookmarks = mBookmarks.values();

    QFile f2( QDir::currentDirPath() + "/.bookmarks" );
    f2.open( IO_WriteOnly );
    QDataStream s2( &f2 );
    s2 << bookmarks;
    f2.close();*/
    qDebug("Destructor GuiHelpWindow finished");
}

/**
*function for opening a file
*/
void GuiHelpWindow::openFile()
{
#ifndef QT_NO_FILEDIALOG
    QString fn = QFileDialog::getOpenFileName( QString::null, QString::null, this );
    if ( !fn.isEmpty() )
        browser->setSource( fn );
#endif
}


/**
*function for printing
*/
void GuiHelpWindow::print()
{
#ifndef QT_NO_PRINTER
    QPrinter printer( QPrinter::HighResolution );
    printer.setFullPage(TRUE);
    if ( printer.setup( this ) ) {
        QPainter p( &printer );
        if( !p.isActive() ) // starting printing failed
            return;
        QPaintDeviceMetrics metrics(p.device());
        int dpiy = metrics.logicalDpiY();
        int margin = (int) ( (2/2.54)*dpiy ); // 2 cm margins
        QRect body( margin, margin, metrics.width() - 2*margin, metrics.height() - 2*margin );
        QSimpleRichText richText( browser->text(),
                                  QFont(),
                                  browser->context(),
                                  browser->styleSheet(),
                                  browser->mimeSourceFactory(),
                                  body.height() );
        richText.setWidth( &p, body.width() );
        QRect view( body );
        int page = 1;
        do {
            richText.draw( &p, body.left(), body.top(), view, colorGroup() );
            view.moveBy( 0, body.height() );
            p.translate( 0 , -body.height() );
            p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
                        view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
            if ( view.top()  >= richText.height() )
                break;
            printer.newPage();
            page++;
        } while (TRUE);
    }
#endif
}


/**
*function for the path selection
*@param _path path_location
*/
void GuiHelpWindow::pathSelected( const QString &_path )
{
    browser->setSource( _path );
    if ( mHistory.values().contains(_path) )
        mHistory[ hist->insertItem( _path ) ] = _path;
}

/**
*function for reading history
*/
void GuiHelpWindow::readHistory()
{
    if ( QFile::exists( QDir::currentDirPath() + "/.history" ) ) {
        QFile f( QDir::currentDirPath() + "/.history" );
        f.open( IO_ReadOnly );
        QDataStream s( &f );
        s >> history;
        f.close();
        while ( history.count() > 20 )
            history.remove( history.begin() );
    }
}

/**
*function for reading the bookmarks
*/
void GuiHelpWindow::readBookmarks()
{
    if ( QFile::exists( QDir::currentDirPath() + "/.bookmarks" ) ) {
        QFile f( QDir::currentDirPath() + "/.bookmarks" );
        f.open( IO_ReadOnly );
        QDataStream s( &f );
        s >> bookmarks;
        f.close();
    }
}

/**
*function for chosing history
*@param i number of chosen history
*/
void GuiHelpWindow::histChosen( int i )
{
    if ( mHistory.contains( i ) )
        browser->setSource( mHistory[ i ] );
}

/**
*function for chosing bookmark
*@param i number of chosen bookmark
*/
void GuiHelpWindow::bookmChosen( int i )
{
    if ( mBookmarks.contains( i ) )
        browser->setSource( mBookmarks[ i ] );
}

/**
*function for adding a bookmark
*/
void GuiHelpWindow::addBookmark()
{
    mBookmarks[ bookm->insertItem( caption() ) ] = browser->context();
}

/** 
*Fires signal cancelled
*/
void GuiHelpWindow::slotBackToProgram(){
  emit cancelled();
}

/** 
*Set the helpfile to be displayed
*@param helfFile name of help file
*/
void GuiHelpWindow::setHelpfile(const QString & helpFile) {
 browser->setSource( helpFile );
}


    
