/***************************************************************************
                          guiiniletter.cpp  -  description
                             -------------------
    begin                : So Nov 7 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms ofthe GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guiiniletter.h"
#include "guimain.h"

#include <qtextedit.h>
#include <qpainter.h>
#include <qrect.h>
#include <qsize.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qframe.h>
#include <qcanvas.h>
#include <qpaintdevicemetrics.h>

#include <iostream>


/**
*constructor of GuiIniLetter
*
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiIniLetter::GuiIniLetter( QWidget *parent, GuiMain* guimain, const char *name, HoBIT* hobit) 
 : GuiIniLetterBase(parent,name) {
  _hobit = hobit;
  _main = guimain;
 

      
  _Printer = new QPrinter(QPrinter::ScreenResolution);
  _Canvas = new QCanvas(iniLetterFrame->width(),iniLetterFrame->height());
  _View = new QCanvasView(_Canvas, iniLetterFrame);
  _View->show();	
  
  _Text = NULL;
  _Font = new QFont ("Courier", 10); 
  _SetupOk = false;
  _ShowPreview = false;
  _LeftBorder = 2.5; //cm
  _TopBorder = 2.5;
  _Text = new QCanvasText(tr("No ini letter available."), _Canvas);
  qDebug("Constructor GuiIniLetter");
}


/**
*destructor of GuiIniLetter
*/
GuiIniLetter::~GuiIniLetter(){
 if(_Printer)
   delete _Printer;
 if(_Canvas)  
   delete _Canvas;
 if(_Font)  
   delete _Font;
 qDebug("Destructor GuiIniletter");
}


/**
*function for printing the IniLetter
*/
void GuiIniLetter::printIniLetter() {
    QWMatrix m, mOld;
    if(_Printer->setup(this)){
        QPainter painter(_Printer);
        QRect rect = painter.viewport();
        QSize size = _Canvas->size();
        size.scale(rect.size(), QSize::ScaleMin);
        painter.setViewport( rect.x(),  rect.y(), size.width(), size.height() );
        painter.setWindow(_Canvas->rect());
        painter.setClipRect(painter.viewport());
        mOld =_View->worldMatrix();
        _View->setWorldMatrix(m);
       QCanvasItemList items = _Canvas->collisions(_Canvas->rect());
       if( items.isEmpty() )
       {
         std::cerr << "\nWarning: PrinterItemList was empty!" << std::endl;

       QCanvasItemList::const_iterator it = items.end();
             while (it != items.begin())
             {
                 it--;
                 (*it)->draw(painter);
                 painter.setPen(black);
                 painter.drawRect((*it)->boundingRect());
             }
       }
      _Canvas->drawArea( _Canvas->rect(), &painter );
      _View->setWorldMatrix(mOld);    
     }
     emit terminated();
}
/** 
*Create a new ini letter and show it on the screen.
*/
void GuiIniLetter::newIniLetter(){

    //comment the next line and uncomment the following test text section, if you want to use the test text
    _Text = new QCanvasText(_hobit->generateIniLetter().c_str(),  _Canvas);
   
   
   //Begin test text generation /////////////////////////////////////////////////
   /*
   QString testText, sLine, sNumber;
   for(int nColumn = 1; nColumn <= 76; nColumn ++){
    sNumber.setNum(nColumn % 10);
    sLine +=sNumber;
   } 
   for (int nRow = 1; nRow < 100; nRow ++){
    sNumber.setNum(nRow);
    testText = testText + sNumber + " " +sLine +"\n";
   }
  _Text->setText(testText);
  */
  //End test text generation//////////////////////////////////////////////////////
  
  _Text->setVisible(true);
  _Text->setFont(*_Font);
  showReadableText();
}

/** 
*Calls the printer setup dialog.
*/
void GuiIniLetter::slotSetupPrinter(){
  _Printer->setup();
 //_main->slotRaiseWidget(HARDWARE_WIDGET);
}

/** 
*Calls the hardware setup dialog.
*/
void GuiIniLetter::slotHardware(){
  emit (addPrinterRequest());
}

/** 
*Displays the text on the canvas, already scaled for printing.
*/
void GuiIniLetter::slotPreview(){
 if(!_SetupOk)
  _SetupOk = _Printer->setup();
  if(_SetupOk){               
    _ShowPreview = ! _ShowPreview; 
    if(_ShowPreview)
         showPreviewText();
    else showReadableText();
  }//end if _SetupOk
}


/**
*function for creating the Layout
*/
void GuiIniLetter::doLayout()
{
        int w, h, space;
	QBoxLayout* framelayout = new QBoxLayout(iniLetterFrame, QBoxLayout::LeftToRight, 0, 0, "framelayout");
        
	w = iniLetterFrame->width();
        h = iniLetterFrame->height();
        space = (w - (int)0.75 * h) /2;
        framelayout->addStretch(space);
        _View->setFixedSize((int)(iniLetterFrame->height() * 0.75), iniLetterFrame->height() -10);
	framelayout->addWidget(_View);	
        framelayout->addStretch(space);
        
}

/** 
*Shows the iniletter text scaled 1:1.
*/
void GuiIniLetter::showReadableText()
{  
    int leftBorder, topBorder; //borders in pixels
    int w, h;
    QRect bounds = _Text->boundingRect();
    QPaintDeviceMetrics metrics(_View);
    leftBorder = (int)((_LeftBorder / 2.54) * metrics.logicalDpiX());
    topBorder = (int)((_TopBorder / 2.54) * metrics.logicalDpiY());
    _Text->setX(leftBorder);
    _Text->setY(topBorder);
    if (metrics.width() > bounds.width())
     w = metrics.width();
    else w = bounds.width(); 
    if (metrics.height() > bounds.height())
     h = metrics.height();
    else h = bounds.height(); 
    _Canvas->resize(w + leftBorder +10, h + topBorder +10); //add a little extra space
    QWMatrix m; //creates an identity matrix
    _View->setWorldMatrix(m);
    _View->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    _View->setLineWidth(4);
    _View->setMidLineWidth(4);
    buttonPreview->setText(tr("&Preview Print"));
    _View->hide();
    _View->show();
}  

/** 
*Shows a scaled print preview.
*/
void GuiIniLetter::showPreviewText(){
    double scale, wPage, wView;
    int leftBorder, topBorder; //borders in pixels
    QPaintDeviceMetrics metrics(_Printer);
    leftBorder = (int)((_LeftBorder / 2.54) * metrics.logicalDpiX());
    topBorder = (int)((_TopBorder / 2.54) * metrics.logicalDpiY());
    _Text->setX(leftBorder);
    _Text->setY(topBorder);
    _Canvas->resize((int)(metrics.width() - 2 * _LeftBorder), (int)(metrics.height() - 2 * topBorder));
    wPage = metrics.width(); //convert to double
    wView = _View->width()-8;
    scale = wView / wPage;
    QWMatrix m (scale, 0.0,    //scaling matrix
                0.0, scale,
                0.0, 0.0);
    _View->setWorldMatrix(m);
    _View->setFrameStyle(QFrame::Box | QFrame::Plain);
    _View->setLineWidth(2);
    _View->setMidLineWidth(2);
    buttonPreview->setText(tr("&End Preview"));
    _View->hide();
    _View->show();
}

/** 
*Renews the layout and sets up the _View according to preview flag.
*@param re pointer on the QResizeEvent - see in QT-Api for more help
*/
void  GuiIniLetter::resizeEvent(QResizeEvent * re){
doLayout();
if(_ShowPreview)
        showPreviewText();
else showReadableText();
}

/** 
*Fires signal cacelled.
*/
void  GuiIniLetter::slotCancel(){
  emit terminated();
}

 
