/***************************************************************************
                          guitransactions.cpp  -  description
                             -------------------
    begin                : Sa Nov 6 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "hobit_transfertemplate.h"
#include "guitransactions.h"
#include <openhbci2/date.h>
#include <openhbci2.h>
#include "aqmoney2/account.h"


#include <qlabel.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <qlineedit.h>

#include "hobit_string.h"

/**
*this is the transaction dialog constructor
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiTransactions::GuiTransactions(QWidget *parent, GuiMain *guimain, const char *name, HoBIT* hobit)
: GuiTransactionsBase(parent,name) {
  _hobit = hobit;
  _main = guimain;
  _LastColumn = -1;
  _Ascending = true;
  _DateFormat = tr("ddd.MMM.d.yyyy", "Date format to be used for transaction list");
  _FromDate = QDate::currentDate();
  _ToDate = QDate::currentDate();
  _TransactionList.setAutoDelete(true);
  connect (tableTransactions->horizontalHeader(), SIGNAL (clicked(int)), this, SLOT(slotSortByColumn(int)));	
  _ExportTable = NULL;
  qDebug("Constructor GuiTransactions");
}


/**
*this is the transaction dialog destructor
*/
GuiTransactions::~GuiTransactions(){
  if(_ExportTable)
      delete _ExportTable;
    
  qDebug("Destructor GuiTransactions");
}


/**Builds the QTable from list.
*/
void GuiTransactions::buildTable(QTable * table, const QString dateFormat, list<Pointer<Transaction> > transactions, bool ascending){

  double income = 0.0;
  double outgo = 0.0;
  int row = 0;
  TransactionItem * item;
  
  tableTransactions->hide();
  tableTransactions->setNumRows(0);
  _TransactionList.clear();
  
  list<Pointer<Transaction> >::iterator  transit;
  if(ascending){ //ascending order
      for (transit = transactions.begin(); transit != transactions.end(); transit++){
        string value = (*transit).ref().value().toString();
      
        double val = (*transit).ref().value().getValue();
        if (val > 0.0) income += val;
        if (val < 0.0) outgo += val;
        item = new TransactionItem( *transit);
        item->addToTable(table, row, dateFormat);
        _TransactionList.append(item);
        row++;   
      }
  }
  else{ //descending order
    transit = transactions.end();
    transit--;
     do{
        string value = (*transit).ref().value().toString();
      
        double val = (*transit).ref().value().getValue();
        if (val > 0.0) income += val;
        if (val < 0.0) outgo += val;
        item = new TransactionItem(*transit);
        item->addToTable(table, row, dateFormat);
        _TransactionList.append(item);
        row++; 
        transit--;  
      }while(transit != transactions.begin());
  }
  
  QString num;
  num.setNum(income,'f', 2);
  lineEditIncome->setText(num + " EUR");
  num.setNum(outgo, 'f', 2);
  lineEditOutGo->setText(num + " EUR");

  tableTransactions->adjustColumn(0);
  tableTransactions->adjustColumn(1);
  tableTransactions->adjustColumn(2);  
  tableTransactions->adjustColumn(3);
  tableTransactions->adjustColumn(4);
  tableTransactions->adjustColumn(5);
  tableTransactions->adjustColumn(6);
  tableTransactions->show();
     
  update();
}



/**
*this is a function for updating the table depending on the selected account
*Add a year to all toDates for "Current..." lists. There may be dates in the future.
*/
void GuiTransactions::slotChangeAccountOrDate(){

    QDate fromdate;
    QDate todate;

    QDate currentDate;
    currentDate = currentDate.currentDate();
    int dateselection = comboBoxDate->currentItem();

    switch (dateselection){

      case 0:  fromdate = QDate(currentDate.year(), 1,1);
               todate = currentDate.addYears(1); //One year later
               qDebug("Show transactions of current year");
               break;
      case 1:  fromdate = QDate(currentDate.year() - 1, 1 ,1);
               todate = QDate(currentDate.year()-1, 12 , 31);
               qDebug("Show transactions of last year");
               break;
      case 2:  fromdate = QDate(currentDate.year(), currentDate.month(), 1);
               todate = currentDate.addYears(1);
               qDebug("Show transactions of current month");
               break;
      case 3:  if(currentDate.month() == 1){ //we are in january, last month is december last year
                fromdate = QDate(currentDate.year()-1, 12 ,1);
                todate = QDate(currentDate.year()-1, 12 ,31);
                qDebug("Show transactions of last month, december last year");
              }
              else{
                fromdate = QDate(currentDate.year(), currentDate.month()-1, 1);
                todate = QDate(currentDate.year(), currentDate.month()-1, 31);
                qDebug("Show transactions of last month");
              }
              break;
      case 4: if (currentDate.month() < 4){
                        fromdate = QDate(currentDate.year(), 1, 1);
                        todate = QDate(currentDate.year() +1, 3 , 31);
              }
              else if (currentDate.month() < 7){
                        fromdate = QDate(currentDate.year(), 4 ,1);
                        todate = QDate(currentDate.year() + 1, 6 , 30);
              }
              else if (currentDate.month() < 10){
                        fromdate = QDate(currentDate.year(), 7 ,1);
                        todate = QDate(currentDate.year() + 1, 9 , 31);
              }
              else if (currentDate.month() <= 12){
                        fromdate = QDate(currentDate.year(), 10 ,1);
                        todate = QDate(currentDate.year() + 1, 12, 31);
              }
              qDebug("Show transactions of current quarter");
              break;
	case 5: if (currentDate.month() < 4){ //last quarter of last year
                          fromdate = QDate(currentDate.year() -1, 10 ,1);
                          todate = QDate(currentDate.year()-1, 12 ,31);
                }
                else if (currentDate.month() < 7){
                          fromdate = QDate(currentDate.year(), 1, 1);
                          todate = QDate(currentDate.year(), 3, 31);
                }
                else if (currentDate.month() < 10){
                          fromdate = QDate(currentDate.year(), 4, 1);
                          todate = QDate(currentDate.year(), 6, 30);
                }
                else if (currentDate.month() <= 12){
                          fromdate = QDate(currentDate.year(), 7, 1);
                          todate = QDate(currentDate.year(), 9, 31);
                }
                qDebug("last quarter");
                break;
     default:  fromdate = QDate(1980, 1,1); //Show all transactions
               todate = currentDate.addYears(1);
               qDebug("Show all transactions");
               break;
       }; //end switch

       qDebug("Transactions from %s to %s", fromdate.toString().latin1(), todate.toString().latin1());
       _FromDate = fromdate;
       _ToDate   = todate;  
       
    buildTable(tableTransactions, _DateFormat,
               _hobit->listTransactions(   0,
                                        "*",
                                        _hobit->getCurrentAccount().ascii(),
                                        "*",
                                        fromdate,
                                        todate), _Ascending);
     update();
}



/**
*this is the slot for raising up the transaction widget
*/
void GuiTransactions::slotGetTransactions(){
      emit getTransactions();
}



/**
*this is the function which executes when the user performs a double click
*on a list item
*
*@param row number of the row 
*@param col number of the col
*@param button number of the button
*@param mousePos mouse-position
*/
void GuiTransactions::slotSelectItem(int row, int col, int button, const QPoint & mousePos){
  TransactionItem* transit =  _TransactionList.at(row);
 
  if (transit){
   Pointer<Transaction> tr = transit->transaction();  
   emit showDetails(tr);
  }
}


/**
*this is a function for a refresh
*/
void GuiTransactions::refresh()
{
  textLabelAccount->setText(_main->getCurrentAccount());
  if (_main->getCurrentAccount() != NULL)
		slotChangeAccountOrDate();
}

/** 
* this is a function for activating GuiTransactionPrinter
*/
void GuiTransactions::slotPrint(){
 emit printTransactions();
}

/** 
* this is a function for reading property of QPtrList<TransactionItem> _TransactionList. 
*/
QPtrList<TransactionItem>  GuiTransactions::getTransactionList()const{
	return _TransactionList;
}

/**
* this is a function for resizing the columns
*
*@param re QResizeEvent
*/
void GuiTransactions::resizeEvent(QResizeEvent * re){
//  if(tableTransactions->columnWidth(2) < 100) //do not make it too narrow
//    tableTransactions->adjustColumn(2);
}

/** Take care of the color for outgoing transactions
*/
void GuiTransactions::paintEvent(QPaintEvent * pe){
  for(int row= 0; row < tableTransactions->numRows(); row++){
    QLabel * labelValue = (QLabel *)tableTransactions->cellWidget(row, 2);
    if(labelValue != NULL){
      QString sValue = labelValue->text();
      if(sValue.find("-", 0) >= 0) //value is negative
          labelValue->setPaletteForegroundColor(Qt::red); //outgo
      else labelValue->setPaletteForegroundColor(Qt::black); //income
    }
  }//end for   

}

/** calls export Dialog */
void GuiTransactions::slotExport(){
    if(_ExportTable)
      delete _ExportTable;
    _ExportTable = new QTable(0,7);
    buildTable(_ExportTable, "yyyy,MM,dd",
               _hobit->listTransactions(   0,
                                        "*",
                                        _hobit->getCurrentAccount().ascii(),
                                        "*",
                                        _FromDate,
                                        _ToDate), 
               _Ascending);
    
    emit exportSelected(_ExportTable);
}

/** Emits helpRequest().
*/
void GuiTransactions::slotHelpRequest(){
 emit helpRequest("transactionlist.html");
}

/** Creates a set of transaction templates from all outgoing transactions in the list.
*/
void GuiTransactions::slotCreateTransferTemplates(){
 QString otherName; 
 QString ourBankCode; 
 QString ourAccountId;   
 QString otherBankCode; 
 QString otherAccountId;
 QStringList purpose; 
 double numericValue =0.0;
 double value =0.0;
 QStringList CreatedTemplates;
 QString otherId;
 bool isDone;
 HoBIT_TransferTemplate * transferTemplate;
 TransactionItem*  transit;
   
 if(QMessageBox::question(this, tr("Question"), 
   tr("MoneyPenny will create a transfer template from every outgoing transaction in the list now.") + "<br>"
   + tr("Continue?"), QMessageBox::Yes, QMessageBox::No)
   == QMessageBox::No)
 return;
 
  //first get the existing templates
 list<Pointer<HoBIT_TransferTemplate> > oldTemplates = _hobit->getTransferTemplates();
 list<Pointer<HoBIT_TransferTemplate> >::iterator it;
 int nTemplate = 0;
 for(it = oldTemplates.begin(); it != oldTemplates.end(); it++){
    nTemplate ++;
    otherAccountId = (*it).ref().otherAccountId();
    otherBankCode = (*it).ref().otherBankCode();
    otherId= otherBankCode + otherAccountId;
    CreatedTemplates.append(otherId);
 }
 qDebug("GuiTransactions::slotCreateTransferTemplates: Number of existing transfer templates: %i",nTemplate);
 nTemplate = 0;
 //now check the Transactionlist for transactions, that are not yet in the template list
 transit = _TransactionList.at(1); 
 for (transit = _TransactionList.first(); transit != NULL; transit = _TransactionList.next()){
      numericValue = transit->getNumericValue();
      otherBankCode = HoBIT_String::toIdFormat(transit->getOtherBankCode());
      otherAccountId = HoBIT_String::toIdFormat(transit->getOtherAccountId());
      purpose = transit->getPurpose();
      if((numericValue < 0.0) && (!otherBankCode.isEmpty()) && (!otherAccountId.isEmpty())){ //this is an outgoing transaction
        otherName = transit->getOtherName().latin1();
        ourBankCode = HoBIT_String::toIdFormat(transit->getOurBankCode());
        ourAccountId = HoBIT_String::toIdFormat(transit->getOurAccountId());
        otherId= otherBankCode + otherAccountId;
        isDone = false;
        QStringList::iterator it;
        for(it = CreatedTemplates.begin(); it != CreatedTemplates.end(); it++){
         if((*it) == otherId)
           isDone = true;
        } 
        if(!isDone && (!otherName.isEmpty()) && (!ourBankCode.isEmpty()) && (!ourAccountId.isEmpty())){
         transferTemplate = 
           new HoBIT_TransferTemplate(ourAccountId,
                                      otherName,
                                      otherAccountId,
                                      otherBankCode,
                                      value,
                                      purpose);
          _hobit->addTransferTemplate(transferTemplate);
          nTemplate++;
          CreatedTemplates.append(QString((otherId)));
        }
      }//end if numericvValue < 0
 }//end for 
 qDebug("GuiTransactions::slotCreateTransferTemplates: Number of tranfer templates added: %i", nTemplate);
 emit refreshRequest();
 QMessageBox::information(this,tr("Information"), tr("The new transfer templates have been added to the list of templates.") );
 emit raiseWidget(TRANSFER_TEMPLATES_WIDGET);
}

  /** Sorts by column. Does not (yet) work for value.
  */
  void GuiTransactions::slotSortByColumn(int column){

    if(column == _LastColumn)
      _Ascending = !_Ascending; //toggle sorting direction
    else _Ascending = true;     //always start in ascending order 
     
    //Sort by text and set the sort arrow
    tableTransactions->sortColumn(column, _Ascending, true); //toggle sorting direction
    _LastColumn = column; 
    //now get a proper order for date sorting
    if(column == 0){ //this creates a table sorted by date (and not by date-text in alphbetical order)
      slotChangeAccountOrDate(); 
    }
    
    //finally adjust all row heights
    for(int nRow = 0; nRow < tableTransactions->numRows(); nRow++)
     tableTransactions->adjustRow(nRow);
 }

