/***************************************************************************
                          listviewitems.cpp  -  description
                             -------------------
    begin                : Mo Nov 15 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "listviewitems.h"
#include <openhbci2/date.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <string>
#include <qlabel.h>

using namespace std;

DatedTransferItem::DatedTransferItem(QTable* table, int row, Pointer<DatedTransfer> dt) {

 _DatedTransfer = dt;
 QString sValue = QString::number(dt.ref().value().getValue(),'f',2); // 2 digits after . "-999.99"
 Date d = dt.ref().date();
 QDate date(d.year(), d.month(), d.day());
 if(table->numRows() < row +1)
   table->setNumRows(row +1);

  _OtherName = new QTableItem(table, QTableItem::Never, QString(dt.ref().otherName().c_str()) );
  _OtherName->setWordWrap(true);
  table->setItem( row, 0, _OtherName);
     
 _Date = new QTableItem(table, QTableItem::Never, date.toString( Qt::LocalDate) );
 _Date->setWordWrap(true);
 table->setItem( row, 1, _Date);
 
 _Purpose = new QTableItem(table, QTableItem::Never, QString(dt.ref().purpose().c_str()) );
 _Purpose->setWordWrap(true);
 table->setItem( row, 2, _Purpose);
   
 _Value = new QTableItem(table, QTableItem::Never, sValue);
 table->setItem( row, 3, _Value);
 table->adjustRow(row);
}

DatedTransferItem::~DatedTransferItem(){
}


/** Encodes the Transaction data as an html- table row string.
  */
QString DatedTransferItem::htmlTableRow(const QString & bgColor){
  QString result;
  Date d = _DatedTransfer.ref().date();
  QDate date(d.year(), d.month(), d.day());
  string value = QString::number(_DatedTransfer.ref().value().getValue(),'f',2).latin1(); // 2 digits after . "-999.99"  
  string ouraccId = _DatedTransfer.ref().accountId();
  string ourbankCode = _DatedTransfer.ref().bankCode();
  string otheraccId = _DatedTransfer.ref().otherAccountId();
  string otherbankCode = _DatedTransfer.ref().otherBankId();
  string othername = _DatedTransfer.ref().otherName();
  string purpose = _DatedTransfer.ref().purpose();

  
 result += "<tr bgcolor =\"" + bgColor + "\">";
   
   result += "<td>";
    result += date.toString( Qt::LocalDate);
   result += "</td>";
   
   result += "<td>";
    result += QString(othername.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(purpose.c_str());
   result += "</td>";

   result += "<td>";
    result += QString(otherbankCode.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(otheraccId.c_str());
   result += "</td>";  
  
   result += "<td align =\"right\">";
    result += QString(value.c_str());
   result += "</td>";
   
 result += "</tr>\n";
 
 return result;
}


/** Read property of DatedTransfer _DatedTransfer. */
const Pointer<DatedTransfer> DatedTransferItem::datedTransfer(){
	return _DatedTransfer;
}


//#####################################################################################################################



//#####################################################################################################################


StandingOrderItem::StandingOrderItem(QTable * table, int row, Pointer<StandingOrder> so){

  _StandingOrder = so;

  string otherName = so.ref().otherName();
  string purp = so.ref().purpose();
  QString sValue = QString::number(so.ref().value().getValue(),'f',2); // 2 digits after . "-999.99"
  if(table->numRows() < row +1)
   table->setNumRows(row +1);
  
  
  _OtherName = new QTableItem(table, QTableItem::Never, QString(so.ref().otherName().c_str()) );
  _OtherName->setWordWrap(true);
  table->setItem( row, 0, _OtherName);
   
  _Purpose = new QTableItem(table, QTableItem::Never, QString(so.ref().purpose().c_str()) );
  _Purpose->setWordWrap(true);
  table->setItem( row, 1, _Purpose);
  
  _Value = new QTableItem(table, QTableItem::Never, sValue + " ");
  _Value->setWordWrap(true);
  table->setItem( row, 2, _Value);
  
 
  QString day(QString::number(_StandingOrder.ref().day()));
  QString turnus(QString::number(_StandingOrder.ref().turnus()));
  QString unit = _StandingOrder.ref().unit();
  QString trUnit;
  if(turnus.toInt() == 1){
     if(unit.find("W") >=0)
       trUnit= QObject::tr("Week");
     else  if(unit.find("M") >=0)
       trUnit= QObject::tr("Month");
     else trUnit = unit + "??";
  }
  else {
     if(unit.find("W") >=0)
       trUnit= QObject::tr("Weeks");
     else  if(unit.find("M") >=0)
       trUnit= QObject::tr("Months");
     else trUnit = unit + "??";
  }
  QString trDay = day;
  if(unit.find("W") >=0)
    trDay= QDate::longDayName(day.toInt());

  _Day = new QTableItem(table, QTableItem::Never, trDay + " ");
  _Day->setWordWrap(false);
  table->setItem( row, 3, _Day);

  _Interval = new QTableItem(table, QTableItem::Never," "+ turnus + " " + trUnit);
  _Interval->setWordWrap(false);
  table->setItem( row, 4, _Interval);
}

StandingOrderItem::~StandingOrderItem(){
}

/** Read property of DatedTransfer _datedTransfer. */
const Pointer<StandingOrder> StandingOrderItem::standingOrder(){
	return _StandingOrder;
}



 /** Encodes the Transaction data as an html- table row string.
  */
QString StandingOrderItem::htmlTableRow(const QString & bgColor){
  
  QString result;
  Date d = _StandingOrder.ref().date();
  QDate date(d.year(), d.month(), d.day());
  d = _StandingOrder.ref().firstDate();
  QDate firstdate(d.year(), d.month(), d.day());
  d = _StandingOrder.ref().lastDate();
  QDate lastdate(d.year(), d.month(), d.day());
  QString day(QString::number(_StandingOrder.ref().day()));
  QString turnus(QString::number(_StandingOrder.ref().turnus()));
  string unit = _StandingOrder.ref().unit();
  QString weekly(QString::number(_StandingOrder.ref().weekly()));
  string yearly = _StandingOrder.ref().yearly();
  
  string value = QString::number(_StandingOrder.ref().value().getValue(),'f',2).latin1(); // 2 digits after . "-999.99"  
  string ouraccId = _StandingOrder.ref().accountId();
  string ourbankCode = _StandingOrder.ref().bankCode();
  string otheraccId = _StandingOrder.ref().otherAccountId();
  string otherbankCode = _StandingOrder.ref().otherBankId();
  string othername = _StandingOrder.ref().otherName();
  string purpose = _StandingOrder.ref().purpose();

  
 result += "<tr bgcolor =\"" + bgColor + "\">";
   
   result += "<td>";
    result += date.toString( Qt::LocalDate);
   result += "</td>";
   
   result += "<td>";
    result += QString(othername.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(purpose.c_str());
   result += "</td>";

   result += "<td>";
    result += QString(otherbankCode.c_str());
   result += "</td>";
   
   result += "<td>";
    result += QString(otheraccId.c_str());
   result += "</td>";  
  
   result += "<td align =\"right\">";
    result += QString(value.c_str());
   result += "</td>";
   
 result += "</tr>\n";
 
 return result;
}

/**********************HoBIT_ListViewItem************************************/
 HoBIT_ListViewItem:: HoBIT_ListViewItem ( QListView * parent,
   QString label1, QString label2, QString label3, QString label4, 
   QString label5, QString label6, QString label7, QString label8):
   QListViewItem (parent, label1, label2, label3, label4, label5, label6, label7, label8)
{
    _LineHeight = parent->fontMetrics().height(); //get it here, because parent may be lost
}
 
 HoBIT_ListViewItem::~HoBIT_ListViewItem (){
 }
 
 /**This computes the multiline height
 */
 void HoBIT_ListViewItem::setup(){
   widthChanged();
   int nLines = 1;
   for(int nColumn = 0; nColumn < 8; nColumn++){
     QString columnText= text(nColumn);
     if(columnText != QString::null){
       QStringList lines = QStringList::split("\n", columnText);
       int columnLines = lines.count();
       if(nLines < columnLines)
         nLines = columnLines;
     }          
   }
   setHeight((nLines * _LineHeight / 2) * 2); //force an even height
 }
